package com.jiuqi.util;

import java.util.GregorianCalendar;
import java.util.Calendar;

public class IDCard
{
    private static final int[] WEIGHS;
    private static final char[] IDCS;
    
    static {
        WEIGHS = new int[] { 7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2, 1 };
        IDCS = new char[] { '1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2' };
    }
    
    private static boolean isNumber(final String s) {
        if (s != null && s.length() > 0) {
            for (int i = 0; i < s.length(); ++i) {
                final char c = s.charAt(i);
                if (c < '0' || c > '9') {
                    return false;
                }
            }
            return true;
        }
        return false;
    }
    
    private static boolean isIDCardChar(final String s) {
        if (s != null && (s.length() == 15 || s.length() == 18)) {
            for (int i = 0; i < s.length(); ++i) {
                final char c = s.charAt(i);
                if ((c < '0' || c > '9') && c != 'X') {
                    return false;
                }
            }
            return true;
        }
        return false;
    }
    
    private static Calendar IdCard15Date(final String s) {
        if (s != null && s.length() == 15) {
            final String birthday = s.substring(6, 12);
            if (isNumber(birthday)) {
                final int year = 1900 + Integer.parseInt(birthday.substring(0, 2));
                final int month = Integer.parseInt(birthday.substring(2, 4));
                final int day = Integer.parseInt(birthday.substring(4, 6));
                if (year < 1900 || year >= 2000 || month < 1 || month > 12 || day < 1 || day > 31) {
                    return null;
                }
                final GregorianCalendar calendar = new GregorianCalendar();
                switch (month) {
                    case 4:
                    case 6:
                    case 9:
                    case 11: {
                        if (day > 30) {
                            return null;
                        }
                        break;
                    }
                    case 2: {
                        if (calendar.isLeapYear(year)) {
                            if (day > 29) {
                                return null;
                            }
                            break;
                        }
                        else {
                            if (day > 28) {
                                return null;
                            }
                            break;
                        }
                        break;
                    }
                }
                calendar.set(1, year);
                calendar.set(2, month - 1);
                calendar.set(5, day);
                return calendar;
            }
        }
        return null;
    }
    
    private static Calendar IdCard18Date(final String s) {
        if (s != null && s.length() == 18) {
            final String birthday = s.substring(6, 14);
            if (isNumber(birthday)) {
                final int year = Integer.parseInt(birthday.substring(0, 4));
                final int month = Integer.parseInt(birthday.substring(4, 6));
                final int day = Integer.parseInt(birthday.substring(6, 8));
                if (year < 1900 || year >= 3000 || month < 1 || month > 12 || day < 1 || day > 31) {
                    return null;
                }
                final GregorianCalendar calendar = new GregorianCalendar();
                final long currTime = calendar.getTimeInMillis();
                switch (month) {
                    case 4:
                    case 6:
                    case 9:
                    case 11: {
                        if (day > 30) {
                            return null;
                        }
                        break;
                    }
                    case 2: {
                        if (calendar.isLeapYear(year)) {
                            if (day > 29) {
                                return null;
                            }
                            break;
                        }
                        else {
                            if (day > 28) {
                                return null;
                            }
                            break;
                        }
                        break;
                    }
                }
                calendar.set(1, year);
                calendar.set(2, month - 1);
                calendar.set(5, day);
                if (calendar.getTimeInMillis() > currTime) {
                    return null;
                }
                return calendar;
            }
        }
        return null;
    }
    
    public static char IDCardC(final String s) {
        int index = 0;
        char result = '\0';
        if (s == null || s.length() != 17) {
            return result;
        }
        for (int i = 0; i < s.length(); ++i) {
            index += (s.charAt(i) - '0') * IDCard.WEIGHS[i];
        }
        index %= 11;
        result = IDCard.IDCS[index];
        return result;
    }
    
    public static boolean isIDCard(final String s) {
        boolean result = false;
        if (s != null && isIDCardChar(s)) {
            if (s.length() == 18) {
                result = (IdCard18Date(s) != null);
            }
            else if (s.length() == 15) {
                result = (IdCard15Date(s) != null);
            }
        }
        return result;
    }
    
    public static String IDCard18(final String s) {
        if (!isIDCard(s)) {
            return null;
        }
        if (s.length() == 15) {
            final StringBuffer result = new StringBuffer(s);
            result.insert(6, "19");
            result.append(IDCardC(result.toString()));
            return result.toString();
        }
        return s;
    }
    
    public static Calendar getIDCardDate(final String s) {
        Calendar result = null;
        if (s != null && isIDCardChar(s)) {
            if (s.length() == 18) {
                result = IdCard18Date(s);
            }
            else if (s.length() == 15) {
                result = IdCard15Date(s);
            }
        }
        return result;
    }
    
    public static void main(final String[] args) {
        System.out.println(getIDCardDate("420802750708151"));
        System.out.println(getIDCardDate("430506200605091512"));
    }
}
